"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_1 = require("graphql");
/**
 *
 * @param f
 * @param xs
 *
 * applies a map function to each member of an array
 * returns non-null results in a new array
 *
 */
function filterMap(f, xs) {
    return xs.reduce(function (acc, x) {
        var res = f(x);
        if (res !== null) {
            // https://www.measurethat.net/Benchmarks/Show/2336/0/arrayprototypeconcat-vs-spread-operator-vs-push
            // shows `acc.push(res)` is 20% faster than `acc = [ res, ... acc ]`
            // plus in context I see no benefit to treat accumulator as a reference to an immutable array.
            acc.push(res);
        }
        return acc;
    }, []);
}
/**
 *
 * @param info
 *
 * Gets argument definitions for the current resolver.
 *
 */
function getFieldArguments(info) {
    var fieldName = info.fieldName, parentType = info.parentType;
    var typeFields = parentType.getFields();
    return typeFields[fieldName].args;
}
/**
 *
 * @param type
 * @param info
 *
 * Finds field arguments of the specfied type
 *
 */
function findArgumentsOfType(type, info) {
    var fieldArguments = getFieldArguments(info);
    return filterMap(function (argDef) {
        var argType = graphql_1.getNamedType(argDef.type);
        if (typeof type === 'string') {
            if (argType.name === type) {
                return argDef;
            }
        }
        else if (argType.name === type.name) {
            return argDef;
        }
        return null;
    }, fieldArguments);
}
exports.findArgumentsOfType = findArgumentsOfType;
/**
 *
 * @param args
 * @param arg
 *
 * Gets the value for argument definition.
 *
 */
function getArgumentValue(args, arg) {
    return args[arg.name];
}
exports.getArgumentValue = getArgumentValue;
/**
 *
 * @param transform
 * @param usual four resolver parameters
 *
 * Higher-order function that produces a map function over argument definitions
 * of a certain type.  Argument values and lists are transformed
 *
 */
function makeArgumentTransform(transform, parent, args, ctx, info) {
    return function (argumentDef) {
        var argumentName = argumentDef.name;
        var argumentValue = getArgumentValue(args, argumentDef);
        info.argument = argumentDef;
        if (Array.isArray(argumentValue)) {
            return Promise.all(argumentValue.reduce(function (acc, elem) {
                if (elem !== undefined && elem !== null) {
                    if (elem instanceof Promise) {
                        return acc.concat([
                            elem.then(function (e) { return transform(e, parent, args, ctx, info); }),
                        ]);
                    }
                    return acc.concat([
                        Promise.resolve(transform(elem, parent, args, ctx, info)),
                    ]);
                }
                return acc;
            }, [])).then(function (newValue) {
                var _a;
                return (_a = {}, _a[argumentName] = newValue, _a);
            });
        }
        if (argumentValue !== undefined && argumentValue !== null) {
            if (argumentValue instanceof Promise) {
                return argumentValue
                    .then(function (e) { return transform(e, parent, args, ctx, info); })
                    .then(function (newValue) {
                    var _a;
                    return (_a = {}, _a[argumentName] = newValue, _a);
                });
            }
            return Promise.resolve(transform(argumentValue, parent, args, ctx, info)).then(function (newValue) {
                var _a;
                return (_a = {}, _a[argumentName] = newValue, _a);
            });
        }
        return null; // exclude arguments when no value provided
        // (undefined is changed to null, because TypeScript)
    };
}
exports.makeArgumentTransform = makeArgumentTransform;
/**
 *
 * @param config
 *
 * A higher-order function which produces a middleware to transform
 * field arguments of a given type.
 *
 */
function processTypeArgs(_a) {
    var type = _a.type, transform = _a.transform;
    return function (resolve, parent, args, ctx, info) {
        var argDefs = findArgumentsOfType(type, info);
        if (argDefs.length) {
            // Apply argument transform function to arguments of the selected type
            // The caller's transform function is applied to values that may be embedded
            // in lists and promises, but not to null or undefined values
            // Finally the resolver is called with transformed argument values
            return (Promise.all(filterMap(makeArgumentTransform(transform, parent, args, ctx, info), argDefs))
                // substitute the transformed values into the args object
                .then(function (newArgs) {
                return newArgs.reduce(function (args, newArg) { return (__assign({}, args, newArg)); }, args);
            })
                .then(function (newArgs) { return resolve(parent, newArgs, ctx, info); }));
        }
        return resolve(parent, args, ctx, info);
    };
}
exports.processTypeArgs = processTypeArgs;
function visitAllArgs(_a) {
    var transform = _a.transform;
    return function (resolve, parent, args, ctx, info) {
        if (!graphql_1.isIntrospectionType(info.parentType)) {
            var argDefs = getFieldArguments(info);
            if (argDefs.length) {
                // Apply argument transform function to all arguments
                // The caller's transform function is applied to values that may be embedded
                // in lists and promises, but not to null or undefined values
                // Finally the resolver is called with transformed argument values
                return (Promise.all(filterMap(makeArgumentTransform(transform, parent, args, ctx, info), argDefs))
                    // substitute the transformed values into the args object
                    .then(function (newArgs) {
                    return newArgs.reduce(function (args, newArg) { return (__assign({}, args, newArg)); }, args);
                })
                    .then(function (newArgs) { return resolve(parent, newArgs, ctx, info); }));
            }
        }
        return resolve(parent, args, ctx, info);
    };
}
exports.visitAllArgs = visitAllArgs;
__export(require("./upload"));
//# sourceMappingURL=index.js.map